# -*- encoding: utf-8 -*-
require 'spec_helper'
#貸手文献

describe ProviderSourcesController do
  before do
    @admin = FactoryGirl.create :admin
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    @user = FactoryGirl.create :user_yas
    @author = @user.author    #ユーザ作成時に連動して作成される
  end
  
  describe '一覧表示に於いて' do
    before do
      sign_in @admin
    end
    context '事前チェックする' do
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :index
        response.should be_success 
      end
      it '@urlsにリストを取得している' do
        get :index
        assigns(:urls).should eq MagicNumber['provider_sources']
      end
      context 'html形式' do
        it 'indexテンプレートを描画する' do
          get :index
          response.should render_template("index")
        end
      end
      context 'json形式' do
      end
    end
    context '管理者権限がないとき' do
      before do
        sign_out @admin
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :index
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :index
          response.should redirect_to '/admins/sign_in'
        end
      end
      context 'json形式' do
      end
    end
  end
  
  describe 'インポートに於いて' do
    before do
      sign_in @admin
      @ps = MagicNumber['provider_sources']
      @results = @ps.map {|u| {u => {'validations' => []}}}
      Provider.stub(:import).with(@ps).and_return @results
    end
    context '事前チェックする' do
      it '貸手モデルに文献からのインポートを依頼している' do
        Provider.should_receive(:import).with(@ps).exactly(1)
        post :import
      end
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        post :import
        response.should be_success 
      end
      it '@resultsにインポート結果を取得している' do
        post :import
        assigns(:results).should eq @results
      end
      context 'html形式' do
        it 'importテンプレートを描画する' do
          post :import
          response.should render_template("import")
        end
      end
      context 'json形式' do
      end
    end
    context '管理者権限がないとき' do
      before do
        sign_out @admin
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          post :import
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          post :import
          response.should redirect_to '/admins/sign_in'
        end
      end
      context 'json形式' do
      end
    end
  end
  

end
